#!/usr/local/bin/bash
# dimlogin.sh
# setting environment variables
# for DIM Distributed Information Managment System
# 
# Author: Peter Zumbruch
# E-Mail: P.Zumbruch@gsi.de
# Created: 25-Oct-2006
#
# Last change on 
#    $Date: 2007/02/07 17:11:20 $ (GMT)
# by 
#    $Author: feldmann $
#
# $RCSfile: dimlogin.sh,v $
# $Date: 2007/02/07 17:11:20 $ (GMT)
# $Revision: 1.1 $
# $State: Exp $

# TODO: 
#       - create tcsh conform

## name of this script
scriptname='dimlogin.sh'

## declare and predefine local variables
dimDir=~/programs/dim
DIM_DNS_NODE=
dim_dns_node=$(hostname).gsi.de
dim_dir=
extraverbose=
lastDir=
os=Linux
osOld=
old_ld_library_path=$LD_LIBRARY_PATH
temporaer=
TMPFILE=
unsourced=
verbose=
version=

## trapping signal to clean up TMPFILE if existing
trap '[ $TMPFILE ] && [ -f $TMPFILE ] && rm -f $TMPFILE' 1 2 3 6 7 9 15

## function definition
#  - help function

printHelp ()
{
  echo 'dimlogin.sh -[dnvh]'
  echo '-- sets environment variables for DIM'
  echo '-- optional options:'
  echo '   -d ARG      set base directory of dim to ARG, verifies valid directory' 
  echo '               [ default: '$dimDir' ]' 
  echo '   -h          prints this help' 
  echo '   -n ARG      set DIM_DNS_NODE to to ARG' 
  echo '               [ default: '$dim_dns_node' ]' 
  echo '   -v          verbose mode' 
  echo '   -vv         extraverbose mode prints output of setup.sh' 
  echo 
  echo "-- you have to call this script with a leading '. '"
  echo "--> i.e.: \'. $scriptname\'"

  # exiting
  if [ $unsourced ]; 
  then 
    exit 1; 
  else 
    return 1; 
  fi
}

## sourced check
#  - sourced call of this routine? ("leading .") ?
#    - if not print help
if [ ${0##*/} = $scriptname ]; 
then 
  unsourced=TRUE;
  printHelp
fi

## get command options
while [ $# -gt 0 ] 
do
   Option=$1 
   OptArg=$2
   case $Option in 
    -d ) # dim base directory '-d dir'
         if [ -z ${OptArg##-*} ];
    	 then 
           echo \'. $scriptname $*\': argument needed for option -d !
           printHelp; 
           return;
         else
           dim_dir=$OptArg
         fi
         if [ ! -d $dim_dir ]; 
         then 
           echo \'. $scriptname $*\': \'$dim_dir\' does not exists ... exiting; 
           return
         fi
         shift;
         shift;
         ;;   
    -h ) # help
         printHelp; 
    	 return 
         shift;
         ;;
    -n ) # dim dns node '-n node name'
         if [ -z ${OptArg##-*} ];
    	 then 
           echo \'. $scriptname $*\': argument needed for option -n !
           printHelp; 
           return;
         fi;
         dim_dns_node=$OptArg;
         shift;
         shift;
         ;;   
    -v ) # verbosity
         verbose=TRUE;
	 shift;;
    -vv )# increased verbosity
         verbose=TRUE;
         extraverbose=TRUE;
	 shift;;
     * ) # other arguments 
         temporaer="$temporaer $Option";
         shift;
         ;;
   esac
done

## Name of DIM DNS Server
#   set to calling host if not set via option '-n NODE' 

DIM_DNS_NODE=$dim_dns_node
export DIM_DNS_NODE

## setting the local base directory variable of dim
if [ ! -z $dim_dir ] 
then
	dimDir=$dim_dir
fi

## Section: "call of original setup bash script for dim"

# - check if setup script exists in dimDir directory
if [ ! -f $dimDir/setup.sh ]; 
then
   echo \'. $scriptname $*\': $dimDir doesn\'t contain the script 'setup.sh'
   return 1
fi

# - OS:
#    - unfortuntately GSI uses also the ENVIRONMENT variable OS
#      --> so, store it, replace it and after call restore it 
#          (HOPING ITS NOT NEEDED FURTHERMORE by DIM)

if [ ! -z $OS ]
then
	os_old=$OS
	OS=$os 
fi 

# - change to directory of DIM (optional parameter dim_dir)
#     - needed because the script takes 
#       'current working directory $(pwd) as basis' 
#       therefore intermediately store current directory in $lastDir

lastDir=$PWD
cd $dimDir

# - create temporary file for the output of the setup.sh file
#     - exit if not succesful

TMPFILE=$(mktemp -q)
if [ $? -ne 0 ]
then
  echo \'. $scriptname $*\': "failed creating tempfile via \'mktemp\' ... exiting."
  return 1
else

# - source setup script and write output (stdout and stderr) to $TMPFILE
  . setup.sh > $TMPFILE 2>&1

# - check for return value of setup.sh
#     - in case of errors print out content of the file $TMPFILE
#     - delete $TMPFILE afterwards if not needed by extraverbose mode later

  if [ $? -ne 0 ]
  then
    echo \'. $scriptname $*\': "error in setup.sh source by this script, output follows ... exiting."
    cat $TMPFILE
    [ -f $TMPFILE ] && rm -f $TMPFILE 
    return 1
  else  
    [ ! $extraverbose ] && [ -f $TMPFILE ] && rm -f $TMPFILE 
  fi
fi 

# - change back to previous values of working directory and $OS 
cd $lastDir

if [ ! -z $osOld ]
then
	OS=$osOld
	export OS
fi 

## clean LD_LIBRARY_PATH from doubles
LD_LIBRARY_PATH=$(echo ${LD_LIBRARY_PATH} | ~epics/bin/uniqpaths)
export LD_LIBRARY_PATH

## verbose mode
#  - verbose
if [ "TRUE" = "$verbose" ] 
then
   echo "---------------"
   echo $scriptname:
   echo -n "DIM_DNS_NODE set to "
   echo ${DIM_DNS_NODE}
   echo -n "LD_LIBRARY_PATH prepended by "
   echo ${LD_LIBRARY_PATH%%$(echo $old_ld_library_path | ~epics/bin/uniqpaths)}
   echo -n "DIMDIR set to "
   echo ${DIMDIR}
   echo some aliases set:
   alias | grep $DIMDIR/$ODIR 

   #  - extra verbosity
   if [ "TRUE" = "$extraverbose" ]
   then
      if [ -f $TMPFILE ]
      then 
         echo --- output of setup.sh:
         cat $TMPFILE
         rm -f $TMPFILE 
      fi

   fi
   echo "---------------"
fi

# delete local variables
unset lastDir osOld TMPFILE old_ld_library_path
unset verbose extraverbose temporaer Option 
unset scriptname account unsourced dim_dns_node dim_dir
