#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "dic.h"

void readCallback();

#define TIMEOUT 10

#ifdef DEBUG
int debug = 1;
#else
int debug = 0;
#endif

char nolink[] = "NO LINK!!!";

int received = 0;
char cmndName[132] = {'\0'};
char svcName[132] = {'\0'};

int svcId = -1;

void commandSent(tag, code)
int *tag;
int *code;
{
    int silent;

    silent = *tag;
    if (!silent)
        dim_print_date_time();
    if (*code)
    {
        if (!silent)
            printf(" Command %s Successfully Sent\n", cmndName);
    }
    else
    {
        if (!silent)
        {
            fprintf(stderr, "In In line: %i   - ", __LINE__ + 1);
            fprintf(stderr, " Command %s Unsuccessfully Sent\n", cmndName);
        }
    }
    received = 1;
    #ifdef WIN32

    wake_up();
    #endif
}



int main(int argc, char **argv)
{
    int i;
    int silent = 0;
    char data[1024] = {'\0'};
    char dns_node[128], *ptr;
    int dns_port = 0;

    dns_node[0] = '\0';

    /*loop (all args)
     * {
     * 		search for "dns" and/or "-s", set appropriate variables;
     * 		
     * 		fill arrays for command name, service name, data, dns port and dns address if possible
     * 		{
     * 			case1: "$> progName cmndName svcName
     * 			->should (could) be a read, but cmndName is prompted later, so only valid cmnds will be executed 
     * 			case2: "$> progName cmndName svcName data
     * 			->should (could) be a write command
     * 		}
     * 		if( command is read )
     * 			do dic_info_service
     * 		else
     * 			do command X per dic_cmnd_callback
     * }*/


    for (i = 1; i < argc; i++)
    {
        if (0 == strcmp(argv[i], "-dns"))
        {
            strcpy(dns_node, argv[i + 1]); /*dns_node = arg behind "-dns", e.g. "dnsNodeName"*/
            if ((ptr = strchr(dns_node, ':'))) /*if there is no ":" in dns_node*/
            {
                *ptr = '\0';
                ptr++;
                sscanf(ptr, "%d", &dns_port); /*searches dns_port number*/
            }
            i++; /*skip next arg, which is "dnsNodeName" and has been copied already*/
        }
        else /*if no "-dns" exists in the nodeName*/
        {
            if (0 == strcmp(argv[i], "-s")) /*search for "-s" arg*/
            {
                silent = 1;
            }
            else /*-dns and -s options have been processed*/
            {
                if ('\0' == cmndName[0]) /*in this step the command name is found, beause it is the next arg as to the specification*/
                {
                    strncpy(cmndName, argv[i], 132 );
                    if ( debug )
                        printf("cmndName: %s\n", cmndName);
                }
                else /*next step: cmndName has been found*/
                {
                    if ( '\0' == svcName[0])
                    {
                        /*in this case the next step is to get the service name*/
                        strcpy( svcName, argv[i] );
                        if ( debug )
                            printf("svcName: %s\n", svcName);
                    }
                    else
                    {
                        if ( '\0' == data[0] )
                        {
                            strcpy(data, argv[i]);
                            if ( debug )
                                printf("data: %s\n", data);
                        }
                    }
                }
            }
        }
    }

    if (dns_node[0])
    {
        dic_set_dns_node(dns_node);
    }
    if (dns_port)
    {
        dic_set_dns_port(dns_port);
    }
    if (!cmndName[0])
    {
        fprintf(stderr, "dim_send_command: Insufficient parameters\n");
        printf("usage: dimCommandLineClient <cmnd_name> <variable_name> [<data>] [-dns <dns_node>] [-s]\n");
        exit( -1);
    }
    if (!svcName[0])
    {
        fprintf(stderr, "dim_send_command: Insufficient parameters\n");
        printf("usage: dimCommandLineClient <cmnd_name> <variable_name> [<data>] [-dns <dns_node>] [-s]\n");
        exit( -1);
    }
    if (!data[0])
        data[0] = '\0';


    if ( !strcmp( cmndName, "read" ) )
    {
        /*do a read per dic_info_service*/
        svcId = dic_info_service_stamped( &svcName[0], ONCE_ONLY, TIMEOUT, 0, 0, readCallback,
                                          9999,   /*tag. is not needed, callback shall only display the data, no need and no means to get the real svc*/
                                          &nolink, strlen(nolink) * sizeof(char));
        if ( debug )
            printf("nach dic_info: svcID: %i, name: '%s'\n", svcId, svcName);
    }
    else
    {
        /*requests cmnd from server*/
        /*cmndName has to be commandServiceName, temporarily just called name,
         * "cmndName:svcName",
         * because each command has to be unique, with regards to the name server,
         * though most values will have a write command, which performs the same action on
         * every service.
         * Therefore you have to distinguish a unique command service,
         * which you can call from the command line, and a command action,
         * e.g. write, which you actually want to be performed on the server.*/
        char *realName;
        
        realName = (char*) calloc(strlen(cmndName) + strlen(":") + strlen(svcName) + 1, sizeof(char) );
        sprintf( realName, "%s:%s", cmndName, svcName );
        if ( debug )
            printf("at client cmnd_callback, in line %i   - ", __LINE__ + 1);
        if ( debug )
            printf("realName: %s , data: %s , pData: %p\n", realName, data, data);
        dic_cmnd_callback(realName, data, strlen(data) + 1, commandSent, silent); /*like dic_cmnd_svc, but executes rout when cmnd has finished*/
        free( realName );
    }

    while (!received)
    {
        if ( debug )
            printf("DIM waits until received\n");
        dim_wait();
    }
    sleep(1);
    return (0);
}

void readCallback(int * tag, char * buffer, int * size)
{
    /*A timestamp functionality would have been nice.
     * Feel free to implement this if you want to,
     * it only consists of secs, millisecs, ack and dic_get_timestamp.*/
    int * secs = (int*) calloc (1, sizeof(int));
    int * millisecs = (int*) calloc (1, sizeof(int));
    int ack;

    if ( NULL == buffer )
    {
        fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
        fprintf(stderr, "Error: readCallback: buffer is NULL\n");
        return ;
    }
    if ( (0 >= *size) || (1024*64) < *size )
    {
        fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
        fprintf(stderr, "Error: readCallback: size of the buffer is out of bounds: 0 < buffer < 1024*64. buffer = %i .\n", *size);
        return ;
    }

    char* tmp = (char*) calloc ((*size) + 1, sizeof(char));
    if (NULL == tmp)
    {
        fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
        fprintf(stderr, "Error: readCallback: couldn't allocate memory for copying buffer into array.\n");
        fflush(stderr);
        return ; /*or go on and try printing buffer nevertheless??*/
    }
    strncpy(tmp, buffer, (*size) + 1);

    if ( debug )
        printf("tag = %i. buffer = %s. size = %i.\n", *tag, buffer, *size);

    ack = dic_get_timestamp( svcId, &secs, &millisecs );
    /*if you touch sec or millisec anywhere from here on, prog will crash*/

    if ( *tag == 9999) /*should not be neccessary*/
    {
        if (debug)
        {
            printf("read received. Value: ");
        }
        printf("%s\n", tmp);

        if ( 0 == strncmp( buffer, nolink, strlen(nolink)) )
        {
            fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
            fprintf(stderr, "NO LINK!!!\n");
        }
        /*if( debug ) printf("ack=%i, sec=%i\n", ack, *secs);*/
        /*I had to comment this out or the prog would crash here!*/
    }
    else
    {
        fprintf(stderr, "In line: %i   - ", __LINE__ + 1);
        fprintf(stderr, "Read failed, wrong tag for service from dic_info_service\n");
    }
    if (ack == 1)
    {
        if ( debug )
            fprintf(stderr, "Here is the place to implement your time stamp! I tried but failed, so it is up to you going nuts...\n");
        /*printf("timestamp (Time in seconds since January 1, 1970): %i,%i secs.\n", *secs, *millisecs);*/
        /*int minutes = ((*secs+(*millisecs/1000))/60);
        int minutesRest = ((*secs+(*millisecs/1000))%60);
        int hours = minutes/60;
        int hoursRest = minutes%60;
        int days = hours/24;
        int daysRest = hours%24;
        int months = days/12;
        int years = days/356;

        int currentYear = 1970 + years;
        printf("is about %i.%i.%i, %i:%i\n", currentYear, months, daysRest, hoursRest, minutesRest);*/
        /*does not work properly, crashes silently*/
    }
    else
        fprintf(stderr, "error: dic_get_timestamp returned %i\n", ack);

    received = 1;
}

